{$I+}
{$R-}
{$A-}
Unit LCore2;
Interface
Uses
    SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
    Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, LMath;
Type
    TResult = Record
        ResStr: String;
        ErrCode: Word;
        ErrStart, ErrEnd: Integer;
    End;
    TVariable = Record
        Name: String[40];
        Value: Extended;
    End;
Var
    Vars: Array [0..100] Of TVariable;
    VarCount, LastVarCount: Word;
    Result: TResult;

Procedure Solve(Expression: String; UsingDegrees_: Boolean);
Function GetVarCount: Word;
Procedure SetVarCount(VarC: Word);
Procedure SetUserCC(CC: Word);

Implementation
Type
    TTypes = (TEOE, TOpenBr, TCloseBr, TDigit, TVar, TSymbOp, TOp,
    TUnOp, TFunction, TUFunc, TUnKnownText, TUnKnownSymb, TBadDigit,
    TSpace, TComma, TBadExtended, TString, TOpCount, TInstr);
    TToken = Record
        TokType: TTypes;
        TokPtr: Pointer;
    End;
    TInstruction = Object
        Name: String[20];
        Code: Byte;
        Procedure Init;
    End;
    TSymbOperator = Object
        Name: String[20];
        Priority, Code: Byte;
        Procedure Init;
        Procedure Execute;
    End;
    TOperator = Object
        Name: String[20];
        Priority, Code: Byte;
        Procedure Init;
        Procedure Execute;
    End;
    TUnaryOperator = Object
        Name: String[20];
        Priority, Code: Byte;
        Procedure Init;
        Procedure Execute;
    End;
    TFunc = Object
        Name: String[10];
        Priority, Code, OpCount: Byte;
        Angle: Boolean;
        Procedure Init;
        Procedure Execute;
    End;
    TApp = Object
        Expr, TempStr: String;
        P, State, Temp: Word;
        TI, STI, OPI: Integer;
        Instr: Array [0..10] Of TInstruction;
        SymbOp: Array [0..20] Of TSymbOperator;
        Op: Array [0..20] Of TOperator;
        UnOp: Array [0..20] Of TUnaryOperator;
        Func: Array [0..15] Of TFunc;
        Imm: Array [0..255] Of Extended;
        OpStack: Array [0..200] Of ShortInt;
        Token, TokStack: Array [0..255] Of TToken;
        ResToken, TempToken: TToken;
        Assignment, UsingDegrees, UsingRPol: Boolean;
        SymbOpCount, OpCount, UnOpCount, StrCount, ImmCount,
        InstrCount, FuncCount, LastVarCount, UserCC: Word;
        LongOp, LongOp2: LongInt;
        TempExt: Extended;
        ChangedVarPtr: Pointer;
        Function IsBinary: Boolean;
        Function IsBinarySymb: Boolean;
        Function IsUnary: Boolean;
        Function IsInstruction: Boolean;
        Function IsVariable: Boolean;
        Function IsFunction: Boolean;
        Function IsUnKnown: Boolean;
        Procedure GetToken;
        Procedure GetImmediate;
        Procedure GetExtended;
        Procedure GetIdent;
        Procedure GetSymbols;
        Procedure CheckPriority;
        Procedure CheckUnPriority;
        Procedure ConvertToRPol;
        Procedure GetRPolStr;
        Procedure SolveIt;
    End;
    TImmPtr = ^Extended; TVarPtr = ^TVariable; TTokenPtr = ^TToken;
    TOpPtr = ^TOperator; TUnOpPtr = ^TUnaryOperator; TStrPtr = ^String;
    TSymbOpPtr = ^TSymbOperator; TInstrPtr = ^TInstruction;
    TFuncPtr = ^TFunc;
Const
    Digits: Set Of Char = ['0'..'9'];
    HexDigits: Set Of Char = ['0'..'9', 'A'..'F'];
    Alphabet: Set Of Char = ['A'..'Z', ''..'', '_'];
    AlphaDigit: Set Of Char = ['A'..'Z', ''..'', '_', '0'..'9'];
    Symbols: Set Of Char = ['+', '-', '*', '/', '\', '^', '>', '<', '='];
    CCSymbols = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
Var
    App: TApp;

Function GetVarCount: Word;
Begin
    GetVarCount:= VarCount;
End;

Procedure SetVarCount(VarC: Word);
Begin
    VarCount:= VarC;
    LastVarCount:= VarCount;
End;

Procedure SetUserCC;
Begin
    App.UserCC:= CC;
End;

Procedure TApp.ConvertToRPol;
Begin
    P:= 1; State:= 0; ImmCount:= 0; TI:= -1; STI:= -1; OPI:= -1;
    Assignment:= False; Result.ErrCode:= 0;
    Repeat
        GetToken;
        If Result.ErrCode > 0 Then Begin
            VarCount:= LastVarCount;
            Exit;
        End;
        Case State Of
          0:
            Case ResToken.TokType Of
              TEOE:
                Result.ErrCode:= 5;
              TOpenBr:
                Begin
                    Inc(STI); TokStack[STI]:= ResToken;
                End;
              TDigit:
                Begin
                    Inc(TI); Token[TI]:= ResToken;
                    State:= 1;
                End;
              TUnKnownText, TUnKnownSymb:
                Begin
                    If IsInstruction Then Begin
                        {       
                              ,
                            }
                        If TInstrPtr(ResToken.TokPtr)^.Code = 0 Then Begin
                            If (TI = 0) Or (STI >= 0) Then Result.ErrCode:= 4;
                            State:= 3;
                        End;
                    End
                    Else If IsUnary Then
                        CheckUnPriority
                    Else If IsVariable Then Begin
                        Inc(TI); Token[TI]:= ResToken;
                        State:= 1;
                    End
                    Else If IsFunction Then Begin
                        If TFuncPtr(ResToken.TokPtr)^.OpCount = 0 Then Begin
                            Inc(TI);
                            Token[TI].TokType:= TDigit;
                            Token[TI].TokPtr:= Addr(Imm[ImmCount]);
                            Imm[ImmCount]:= 0; Inc(ImmCount);
                            Inc(TI); Token[TI]:= ResToken;
                            State:= 1;
                        End
                        Else Begin
                            Inc(STI); TokStack[STI]:= ResToken;
                            Inc(OPI); OpStack[OPI]:= TFuncPtr(ResToken.TokPtr)^.OpCount;
                            State:= 2;
                        End;
                    End
                    Else If IsUnKnown Then
                        Result.ErrCode:= 3
                    Else
                        Result.ErrCode:= 4;
                End;
              TSpace:;
              TBadDigit:
                Result.ErrCode:= 1;
              TBadExtended:
                Result.ErrCode:= 12;
              Else
                Result.ErrCode:= 4;
            End;
          1:
            Case ResToken.TokType Of
              TEOE:
                Begin
                    While STI >= 0 Do Begin
                        If Not(TokStack[STI].TokType In [TSymbOp, TOp, TUnOp]) Then Begin
                            Result.ErrCode:= 6;
                            Exit;
                        End;
                        Inc(TI);
                        Token[TI]:= TokStack[STI];
                        Dec(STI);
                    End;
                    Break;
                End;
              TCloseBr:
                Begin
                    While (TokStack[STI].TokType <> TOpenBr) And (STI >= 0) Do Begin
                        Inc(TI);
                        Token[TI]:= TokStack[STI];
                        Dec(STI);
                    End;
                    If TokStack[STI].TokType <> TOpenBr Then Result.ErrCode:= 6;
                    Dec(STI);
                    If TokStack[STI].TokType = TFunction Then Begin
                        Dec(OpStack[OPI]);
                        If OpStack[OPI] < 0 Then Result.ErrCode:= 24;
                        If OpStack[OPI] > 0 Then Result.ErrCode:= 23;
                        Inc(TI); Token[TI]:= TokStack[STI];
                        Dec(STI); Dec(OPI);
                    End;
                End;
              TUnKnownText:
                Begin
                    If IsBinary Then Begin
                        CheckPriority;
                        State:= 0;
                    End
                    Else If IsUnKnown Then
                        Result.ErrCode:= 3
                    Else
                        Result.ErrCode:= 4;
                End;
              TUnKnownSymb:
                Begin
                    If IsBinarySymb Then Begin
                        If (TSymbOpPtr(ResToken.TokPtr)^.Code = 5) And (TI = 0)
                        And (STI < 0) And (Token[TI].TokType = TVar) Then Begin
                            Assignment:= True;
                            TempToken:= Token[TI];
                            ResToken.TokPtr:= Addr(SymbOp[12]);
                        End;
                        Begin
                            CheckPriority;
                            State:= 0;
                        End;
                    End
                    Else If IsUnKnown Then
                        Result.ErrCode:= 3
                    Else
                        Result.ErrCode:= 4;
                End;
              TComma:
                Begin
                    While (TokStack[STI].TokType <> TOpenBr) And (STI > -1) Do Begin
                        Inc(TI);
                        Token[TI]:= TokStack[STI];
                        Dec(STI);
                    End;
                    Dec(OpStack[OPI]);
                    State:= 0;
                End;
              TSpace:;
              Else
                Result.ErrCode:= 4;
            End;
          2:
            {  .  }
            Begin
                Case ResToken.TokType Of
                  TOpenBr:
                    Begin
                        Inc(STI);
                        TokStack[STI]:= ResToken;
                        State:= 0;
                    End;
                  TSpace:;
                  Else
                    Result.ErrCode:= 18;
                End;
            End;
          3:
            {  .   Dim}
            Case ResToken.TokType Of
              TUnKnownText:
                Begin
                    If VarCount > 100 Then
                        Result.ErrCode:= 22
                    Else If IsUnKnown Then Begin
                        Inc(VarCount);
                        Vars[VarCount].Name:= TempStr;
                        Vars[VarCount].Value:= 0;
                        State:= 4;
                    End
                    Else
                        Result.ErrCode:= 17
                End;
              TSpace:
                ;
              TEOE:
                Result.ErrCode:= 5;
              Else
                Result.ErrCode:= 4;
            End;
          4:
            {     .  Dim}
            Case ResToken.TokType Of
              TSpace:
                ;
              TEOE:
                Begin
                    LastVarCount:= VarCount;
                    Result.ErrCode:= 25;
                    Exit; {  .   SolveIt  }
                End;
              TComma:
                State:= 3;
              Else
                Result.ErrCode:= 4;
            End;
        End;
    Until False;
    App.SolveIt;
End;

Procedure TApp.GetRPolStr;
Begin
    P:= 1; State:= 0; ImmCount:= 0; TI:= -1; STI:= -1; OPI:= -1;
    Assignment:= False; Result.ErrCode:= 0;
    Repeat
        GetToken;
        If Result.ErrCode > 0 Then Begin
            VarCount:= LastVarCount;
            Exit;
        End;
        Case State Of
          0:
            Case ResToken.TokType Of
              TDigit:
                Begin
                    Inc(TI); Token[TI]:= ResToken;
                End;
              TUnKnownText:
                If IsBinary Or IsUnary Then Begin
                    Inc(TI); Token[TI]:= ResToken;
                End
                Else
                    Result.ErrCode:= 4;
              TUnKnownSymb:
                If IsBinarySymb Or IsUnary Then Begin
                    Inc(TI); Token[TI]:= ResToken;
                End
                Else
                    Result.ErrCode:= 4;
              TEOE:
                Break;
              TSpace:;
              Else
                Result.ErrCode:= 4;
            End;
        End;
    Until False;
    App.SolveIt;
End;

Procedure TApp.GetToken;
Begin
    Result.ErrStart:= P;
    Case Ord(Expr[P]) Of
      13:
        ResToken.TokType:= TEOE;
      32:
        Begin
            Repeat
                Inc(P);
            Until Expr[P] <> ' ';
            ResToken.TokType:= TSpace;
        End;
      38:
        Begin
            Inc(P);
            GetExtended;
        End;
      40:
        Begin
            ResToken.TokType:= TOpenBr;
            Inc(P);
        End;
      41:
        Begin
            ResToken.TokType:= TCloseBr;
            Inc(P);
        End;
      42, 43, 45, 47, 60..62, 92, 94:
        GetSymbols;
      44:
        Begin
            ResToken.TokType:= TComma;
            Inc(P);
        End;
      46, 48..57:
        GetImmediate;
      65..90, 95, 97..122, 192..255:
        GetIdent;
      Else
        Result.ErrCode:= 2;
    End;
    Result.ErrEnd:= P;
End;

Procedure TApp.CheckPriority;
Begin
    While (TokStack[STI].TokType In [TSymbOp, TOp, TUnOp])
    And (TOpPtr(TokStack[STI].TokPtr)^.Priority >=
    TOpPtr(ResToken.TokPtr)^.Priority) Do Begin
        Inc(TI);
        Token[TI]:= TokStack[STI];
        Dec(STI);
    End;
    Inc(STI);
    TokStack[STI]:= ResToken;
End;

Procedure TApp.CheckUnPriority;
Begin
    While (TokStack[STI].TokType In [TSymbOp, TOp, TUnOp])
    And (TOpPtr(TokStack[STI].TokPtr)^.Priority >
    TOpPtr(ResToken.TokPtr)^.Priority) Do Begin
        Inc(TI);
        Token[TI]:= TokStack[STI];
        Dec(STI);
    End;
    Inc(STI);
    TokStack[STI]:= ResToken;
End;

Procedure TApp.SolveIt;
Begin
    P:= 0; STI:= -1;
    Repeat
        If Result.ErrCode > 0 Then Begin
            TImmPtr(TokStack[STI].TokPtr)^:= 0;
            Exit;
        End;
        Case Token[P].TokType Of
          TDigit:
            Begin
                Inc(STI);
                TokStack[STI]:= Token[P];
            End;
          TVar:
            Begin
                {     ,   
                  }
                Inc(STI);
                Imm[ImmCount]:= TVarPtr(Token[P].TokPtr)^.Value;
                TokStack[STI].TokType:= TDigit;
                TokStack[STI].TokPtr:= Addr(Imm[ImmCount]);
                Inc(ImmCount);
            End;
          TSymbOp:
            SymbOp[TOpPtr(Token[P].TokPtr)^.Code].Execute;
          TOp:
            Op[TOpPtr(Token[P].TokPtr)^.Code].Execute;
          TUnOp:
            UnOp[TUnOpPtr(Token[P].TokPtr)^.Code].Execute;
          TFunction:
            Func[TFuncPtr(Token[P].TokPtr)^.Code].Execute
        End;
        Inc(P);
    Until P > TI;
    If STI < 0 Then Begin
        Result.ErrCode:= 9;
        Exit;
    End;
    If STI > 0 Then Begin
       Result.ErrCode:= 16;
       Exit;
    End;
    If Assignment Then
        Result.ResStr:= TVarPtr(ChangedVarPtr)^.Name + '='
        + FloatToStr(TImmPtr(TokStack[STI + 1].TokPtr)^)
    Else
        Result.ResStr:= FloatToStr(TImmPtr(TokStack[STI].TokPtr)^);
End;

Procedure TApp.GetImmediate;
Begin
    If Expr[P] = '.' Then Begin
        ResToken.TokType:= TBadDigit;
        Exit;
    End;
    Temp:= P;
    Repeat
        Inc(P);
    Until Not(Expr[P] In Digits);
    If Expr[P] = '.' Then Repeat
        Inc(P);
    Until Not(Expr[P] In Digits);
    Imm[ImmCount]:= StrToFloat(Copy(Expr, Temp, P - Temp));
    ResToken.TokType:= TDigit;
    ResToken.TokPtr:= Addr(Imm[ImmCount]);
    Inc(ImmCount);
End;

Procedure TApp.GetIdent;
Begin
    Temp:= P;
    Repeat
        Inc(P);
    Until Not(Expr[P] In AlphaDigit);
    TempStr:= Copy(Expr, Temp, P - Temp);
    If Length(TempStr) > 40 Then Result.ErrCode:= 19;
    ResToken.TokType:= TUnKnownText;
End;

Procedure TApp.GetSymbols;
Begin
    Temp:= P;
    Repeat
        Inc(P);
    Until Not(Expr[P] In Symbols);
    TempStr:= Copy(Expr, Temp, P - Temp);
    ResToken.TokType:= TUnKnownSymb;
End;

Procedure TApp.GetExtended;
Var
    TempSet: Set Of Char;
    BaseOfCount: Word;
Begin
    Case Expr[P] Of
      'B':
        Begin
            TempSet:= ['0'..'1'];
            BaseOfCount:= 2;
        End;
      'O':
        Begin
            TempSet:= ['0'..'7'];
            BaseOfCount:= 8;
        End;
      'H':
        Begin
            TempSet:= ['0'..'9', 'A'..'F'];
            BaseOfCount:= 16;
        End;
      'U':
        Begin
            TempSet:= []; TempStr:= CCSymbols;
            For BaseOfCount:= 1 To UserCC Do Include(TempSet, TempStr[BaseOfCount]);
        End;
      Else
        Begin
            ResToken.TokType:= TBadExtended;
            Exit;
        End;
    End;
    Inc(P);
    Temp:= P;
    While Expr[P] In TempSet Do Inc(P);
    If (P - Temp < 1) Or (Expr[P] In Digits + Alphabet) Then Begin
        Result.ErrCode:= 13;
        Exit;
    End;
    Imm[ImmCount]:= OtherCCToExtended(Copy(Expr, Temp, P - Temp), BaseOfCount);
    ResToken.TokType:= TDigit;
    ResToken.TokPtr:= Addr(Imm[ImmCount]);
    Inc(ImmCount);
End;

Procedure TOperator.Execute;
Begin
    With App Do Begin
        Case Code Of
          0..4:
            Begin
                Try
                    Dec(STI);
                    LongOp:= Trunc(TImmPtr(TokStack[STI].TokPtr)^);
                    LongOp2:= Trunc(TImmPtr(TokStack[STI + 1].TokPtr)^);
                Except
                    Result.ErrCode:= 11;
                    Exit;
                End;
                Case Code Of
                  0:
                    TImmPtr(TokStack[STI].TokPtr)^:= LongOp And LongOp2;
                  1:
                    TImmPtr(TokStack[STI].TokPtr)^:= LongOp Or LongOp2;
                  2:
                    TImmPtr(TokStack[STI].TokPtr)^:= LongOp Xor LongOp2;
                  3:
                    TImmPtr(TokStack[STI].TokPtr)^:= LongOp Shl LongOp2;
                  4:
                    TImmPtr(TokStack[STI].TokPtr)^:= LongOp Shr LongOp2;
                End;
            End;
          5:
            Try
                Dec(STI);
                TImmPtr(TokStack[STI].TokPtr)^:=
                TImmPtr(TokStack[STI].TokPtr)^ - Int(TImmPtr(TokStack[STI].TokPtr)^
                / TImmPtr(TokStack[STI + 1].TokPtr)^) * TImmPtr(TokStack[STI + 1].TokPtr)^;
            Except
                On EZeroDivide Do
                    Result.ErrCode:= 7;
                Else
                    Result.ErrCode:= 8;
            End;
        End;
    End;
End;

Procedure TUnaryOperator.Execute;
Begin
    With App Do Begin
        Case Code Of
          0:
            TImmPtr(TokStack[STI].TokPtr)^:= -TImmPtr(TokStack[STI].TokPtr)^;
          1:
            TImmPtr(TokStack[STI].TokPtr)^:= +TImmPtr(TokStack[STI].TokPtr)^;
          2:
            Try
                TImmPtr(TokStack[STI].TokPtr)^:= Not(Trunc(TImmPtr(TokStack[STI].TokPtr)^));

            Except
                Result.ErrCode:= 11;
                Exit;
            End;
        End;
    End;
End;

Procedure TSymbOperator.Execute;
Begin
    With App Do Try
        Case Code Of
          0:
            Begin
                Dec(STI);
                If TImmPtr(TokStack[STI].TokPtr)^ > TImmPtr(TokStack[STI + 1].TokPtr)^ Then
                    TImmPtr(TokStack[STI].TokPtr)^:= 1
                Else
                    TImmPtr(TokStack[STI].TokPtr)^:= 0;
            End;
          1:
            Begin
                Dec(STI);
                If TImmPtr(TokStack[STI].TokPtr)^ < TImmPtr(TokStack[STI + 1].TokPtr)^ Then
                    TImmPtr(TokStack[STI].TokPtr)^:= 1
                Else
                    TImmPtr(TokStack[STI].TokPtr)^:= 0;
            End;
          2:
            Begin
                Dec(STI);
                If TImmPtr(TokStack[STI].TokPtr)^ <> TImmPtr(TokStack[STI + 1].TokPtr)^ Then
                    TImmPtr(TokStack[STI].TokPtr)^:= 1
                Else
                    TImmPtr(TokStack[STI].TokPtr)^:= 0;
            End;
          3:
            Begin
                Dec(STI);
                If TImmPtr(TokStack[STI].TokPtr)^ >= TImmPtr(TokStack[STI + 1].TokPtr)^ Then
                    TImmPtr(TokStack[STI].TokPtr)^:= 1
                Else
                    TImmPtr(TokStack[STI].TokPtr)^:= 0;
            End;
          4:
            Begin
                Dec(STI);
                If TImmPtr(TokStack[STI].TokPtr)^ <= TImmPtr(TokStack[STI + 1].TokPtr)^ Then
                    TImmPtr(TokStack[STI].TokPtr)^:= 1
                Else
                    TImmPtr(TokStack[STI].TokPtr)^:= 0;
            End;
          5:
            Begin
                Dec(STI);
                If TImmPtr(TokStack[STI].TokPtr)^ = TImmPtr(TokStack[STI + 1].TokPtr)^ Then
                    TImmPtr(TokStack[STI].TokPtr)^:= 1
                Else
                    TImmPtr(TokStack[STI].TokPtr)^:= 0;
            End;
          6:
            Begin
                Dec(STI);
                TImmPtr(TokStack[STI].TokPtr)^:= TImmPtr(TokStack[STI].TokPtr)^ + TImmPtr(TokStack[STI + 1].TokPtr)^;
            End;
          7:
            Begin
                Dec(STI);
                TImmPtr(TokStack[STI].TokPtr)^:= TImmPtr(TokStack[STI].TokPtr)^ - TImmPtr(TokStack[STI + 1].TokPtr)^;
            End;
          8:
            Begin
                Dec(STI);
                TImmPtr(TokStack[STI].TokPtr)^:=
                TImmPtr(TokStack[STI].TokPtr)^ * TImmPtr(TokStack[STI + 1].TokPtr)^;
            End;
          9:
            Begin
                Dec(STI);
                TImmPtr(TokStack[STI].TokPtr)^:=
                TImmPtr(TokStack[STI].TokPtr)^ / TImmPtr(TokStack[STI + 1].TokPtr)^;
            End;
          10:
             Begin
                Dec(STI);
                TImmPtr(TokStack[STI].TokPtr)^:=
                Int(TImmPtr(TokStack[STI].TokPtr)^ / TImmPtr(TokStack[STI + 1].TokPtr)^);
            End;
          11:
            Begin
                Dec(STI);
                TImmPtr(TokStack[STI].TokPtr)^:=
                LMath.Power(TImmPtr(TokStack[STI].TokPtr)^, TImmPtr(TokStack[STI + 1].TokPtr)^);
                {Exp(TImmPtr(TokStack[STI + 1].TokPtr)^ * Ln(TImmPtr(TokStack[STI].TokPtr)^));}
            End;
          12:
            Begin
                TVarPtr(TempToken.TokPtr)^.Value:= TImmPtr(TokStack[STI].TokPtr)^;
                ChangedVarPtr:= TempToken.TokPtr;
                Result.ErrCode:= 26;
                Dec(STI);
            End;
        End;
    Except
         On EZeroDivide Do Result.ErrCode:= 7;
         On EOverFlow Do Result.ErrCode:= 20;
         On EUnderFlow Do Result.ErrCode:= 21;
         Else Result.ErrCode:= 8;
    End;
End;

Procedure TFunc.Execute;
Begin
    With App Do Try
        If Angle And UsingDegrees Then TImmPtr(TokStack[STI].TokPtr)^:=
        RadToDeg(TImmPtr(TokStack[STI].TokPtr)^);
        Case Code Of
          0:
            TImmPtr(TokStack[STI].TokPtr)^:= Abs(TImmPtr(TokStack[STI].TokPtr)^);
          1:
            Begin
                If TImmPtr(TokStack[STI].TokPtr)^ < 0 Then Begin
                    Result.ErrCode:= 14;
                    Exit;
                End;
                TImmPtr(TokStack[STI].TokPtr)^:= SqRt(TImmPtr(TokStack[STI].TokPtr)^);
            End;
          2:
            TImmPtr(TokStack[STI].TokPtr)^:= Int(TImmPtr(TokStack[STI].TokPtr)^);
          3:
            TImmPtr(TokStack[STI].TokPtr)^:= Sin(TImmPtr(TokStack[STI].TokPtr)^);
          4:
            TImmPtr(TokStack[STI].TokPtr)^:= Cos(TImmPtr(TokStack[STI].TokPtr)^);
          5:
            TImmPtr(TokStack[STI].TokPtr)^:=
            Sin(TImmPtr(TokStack[STI].TokPtr)^) / Cos(TImmPtr(TokStack[STI].TokPtr)^);
          6:
            TImmPtr(TokStack[STI].TokPtr)^:=
            1 / Sin(TImmPtr(TokStack[STI].TokPtr)^) / Cos(TImmPtr(TokStack[STI].TokPtr)^);
          7:
            Begin
                Randomize;
                TImmPtr(TokStack[STI].TokPtr)^:= Random;
            End;
          8:
            TImmPtr(TokStack[STI].TokPtr)^:= PI;
          9:
            TImmPtr(TokStack[STI].TokPtr)^:= ArcTan(TImmPtr(TokStack[STI].TokPtr)^);
          10:
            TImmPtr(TokStack[STI].TokPtr)^:= Exp(TImmPtr(TokStack[STI].TokPtr)^);
          11:
            TImmPtr(TokStack[STI].TokPtr)^:= Ln(TImmPtr(TokStack[STI].TokPtr)^);
          12:
            TImmPtr(TokStack[STI].TokPtr)^:= RadToDeg(TImmPtr(TokStack[STI].TokPtr)^);
          13:
            TImmPtr(TokStack[STI].TokPtr)^:= DegToRad(TImmPtr(TokStack[STI].TokPtr)^);
        End;
    Except
        On EZeroDivide Do Result.ErrCode:= 7;
        On EOverFlow Do Result.ErrCode:= 20;
        On EUnderFlow Do Result.ErrCode:= 21;
        Else Result.ErrCode:= 8;
    End;
End;

Function TApp.IsBinary: Boolean;
Begin
    Temp:= 0;
    Repeat
        If Op[Temp].Name = TempStr Then Begin
            ResToken.TokType:= TOp;
            ResToken.TokPtr:= Addr(App.Op[Temp]);
            IsBinary:= True;
            Exit;
        End;
        Inc(Temp);
    Until Temp > OpCount;
    IsBinary:= False;
End;

Function TApp.IsBinarySymb: Boolean;
Begin
    While True Do Begin
        Temp:= 0;
        Repeat
            If SymbOp[Temp].Name = TempStr Then Begin
                ResToken.TokType:= TSymbOp;
                ResToken.TokPtr:= Addr(App.SymbOp[Temp]);
                IsBinarySymb:= True;
                Exit;
            End;
            Inc(Temp);
        Until Temp > SymbOpCount;
        If (Length(TempStr) > 1) And (ResToken.TokType = TUnKnownSymb) Then Begin
            Dec(P); Dec(TempStr[0]);
        End
        Else Break;
    End;
    IsBinarySymb:= False;
End;

Function TApp.IsUnary: Boolean;
Begin
    While True Do Begin
        Temp:= 0;
        Repeat
            If UnOp[Temp].Name = TempStr Then Begin
                ResToken.TokType:= TUnOp;
                ResToken.TokPtr:= Addr(App.UnOp[Temp]);
                IsUnary:= True;
                Exit;
            End;
            Inc(Temp);
        Until Temp > UnOpCount;
        If (Length(TempStr) > 1) And (ResToken.TokType = TUnKnownSymb) Then Begin
            Dec(P); Dec(TempStr[0]);
        End
        Else Break;
    End;
    IsUnary:= False;
End;

Function TApp.IsInstruction: Boolean;
Begin
    Temp:= 0;
    Repeat
        If Instr[Temp].Name = TempStr Then Begin
            ResToken.TokType:= TInstr;
            ResToken.TokPtr:= Addr(App.Instr[Temp]);
            IsInstruction:= True;
            Exit;
        End;
        Inc(Temp);
    Until Temp > InstrCount;
    IsInstruction:= False;
End;

Function TApp.IsVariable: Boolean;
Begin
    Temp:= 0;
    Repeat
        If Vars[Temp].Name = TempStr Then Begin
            ResToken.TokType:= TVar;
            ResToken.TokPtr:= Addr(Vars[Temp]);
            IsVariable:= True;
            Exit;
        End;
        Inc(Temp);
    Until Temp > VarCount;
    IsVariable:= False;
End;

Function TApp.IsFunction: Boolean;
Begin
    Temp:= 0;
    Repeat
        If Func[Temp].Name = TempStr Then Begin
            ResToken.TokType:= TFunction;
            ResToken.TokPtr:= Addr(App.Func[Temp]);
            IsFunction:= True;
            Exit;
        End;
        Inc(Temp);
    Until Temp > FuncCount;
    IsFunction:= False;
End;

Function TApp.IsUnKnown: Boolean;
Begin
    If IsBinary Or IsBinarySymb Or IsUnary Or IsInstruction Or
    IsFunction Or IsVariable Then Begin
        IsUnKnown:= False;
        Exit;
    End;
    IsUnKnown:= True;
End;

Procedure TInstruction.Init;
Begin
    With App Do Begin
        Instr[0].Name:= 'DIM';
        Instr[0].Code:= 0;
        InstrCount:= 3;
    End;
End;

Procedure TFunc.Init;
Begin
    With App Do Begin
        Func[0].Angle:= False; Func[1].Angle:= False; Func[2].Angle:= False;
        Func[3].Angle:= True; Func[4].Angle:= True; Func[5].Angle:= True;
        Func[6].Angle:= True; Func[7].Angle:= False; Func[8].Angle:= False;
        Func[9].Angle:= True; Func[10].Angle:= False; Func[11].Angle:= False;
        Func[12].Angle:= False; Func[13].Angle:= False;
        Func[0].Name:= 'ABS'; Func[1].Name:= 'SQR'; Func[2].Name:= 'INT';
        Func[3].Name:= 'SIN'; Func[4].Name:= 'COS'; Func[5].Name:= 'TAN';
        Func[6].Name:= 'CTG'; Func[7].Name:= 'RND'; Func[8].Name:= 'PI';
        Func[9].Name:= 'ARCTAN'; Func[10].Name:= 'EXP'; Func[11].Name:= '';
        Func[12].Name:= 'RAD'; Func[13].Name:= 'DEG';
        Func[0].Code:= 0; Func[1].Code:= 1; Func[2].Code:= 2;
        Func[3].Code:= 3; Func[4].Code:= 4; Func[5].Code:= 5;
        Func[6].Code:= 6; Func[7].Code:= 7; Func[8].Code:= 8;
        Func[9].Code:= 9; Func[10].Code:= 10; Func[11].Code:= 11;
        Func[12].Code:= 12; Func[13].Code:= 13;
        Func[0].Priority:= 251; Func[1].Priority:= 251; Func[2].Priority:= 251;
        Func[3].Priority:= 251; Func[4].Priority:= 251; Func[5].Priority:= 251;
        Func[6].Priority:= 251; Func[7].Priority:= 251; Func[8].Priority:= 251;
        Func[9].Priority:= 251; Func[10].Priority:= 251; Func[11].Priority:= 251;
        Func[12].Priority:= 251; Func[13].Priority:= 251;
        Func[0].OpCount:= 1; Func[1].OpCount:= 1; Func[2].OpCount:= 1;
        Func[3].OpCount:= 1; Func[4].OpCount:= 1; Func[5].OpCount:= 1;
        Func[6].OpCount:= 1; Func[7].OpCount:= 0; Func[8].OpCount:= 0;
        Func[9].OpCount:= 1; Func[10].OpCount:= 1; Func[11].OpCount:= 1;
        Func[12].OpCount:= 1; Func[13].OpCount:= 1;
        FuncCount:= 13;
    End;
End;

Procedure TUnaryOperator.Init;
Begin
    With App Do Begin
        UnOp[0].Name:= '-'; UnOp[1].Name:= '+'; UnOp[2].Name:= 'NOT';
        UnOp[0].Priority:= 250; UnOp[1].Priority:= 250; UnOp[2].Priority:= 250;
        UnOp[0].Code:= 0; UnOp[1].Code:= 1; UnOp[2].Code:= 2;
        UnOpCount:= 3;
    End;
End;

Procedure TSymbOperator.Init;
Begin
    With App Do Begin
        SymbOp[0].Name:= '>'; SymbOp[1].Name:= '<'; SymbOp[2].Name:= '<>';
        SymbOp[3].Name:= '>='; SymbOp[4].Name:= '<='; SymbOp[5].Name:= '=';
        SymbOp[6].Name:= '+'; SymbOp[7].Name:= '-'; SymbOp[8].Name:= '*';
        SymbOp[9].Name:= '/'; SymbOp[10].Name:= '\'; SymbOp[11].Name:= '^';
        SymbOp[12].Name:= '';
        SymbOp[0].Priority:= 2; SymbOp[1].Priority:= 2; SymbOp[2].Priority:= 2;
        SymbOp[3].Priority:= 2; SymbOp[4].Priority:= 2; SymbOp[5].Priority:= 2;
        SymbOp[6].Priority:= 3; SymbOp[7].Priority:= 3; SymbOp[8].Priority:= 4;
        SymbOp[9].Priority:= 4; SymbOp[10].Priority:= 4; SymbOp[11].Priority:= 5;
        SymbOp[12].Priority:= 0; SymbOp[0].Code:= 0; SymbOp[1].Code:= 1;
        SymbOp[2].Code:= 2; SymbOp[3].Code:= 3; SymbOp[4].Code:= 4;
        SymbOp[5].Code:= 5; SymbOp[6].Code:= 6; SymbOp[7].Code:=7;
        SymbOp[8].Code:= 8; SymbOp[9].Code:= 9; SymbOp[10].Code:= 10;
        SymbOp[11].Code:= 11; SymbOp[12].Code:= 12;
        SymbOpCount:= 16;
    End;
End;

Procedure TOperator.Init;
Begin
    With App Do Begin
        Op[0].Name:= 'AND'; Op[1].Name:= 'OR'; Op[2].Name:= 'XOR';
        Op[3].Name:= 'SHL'; Op[4].Name:= 'SHR'; Op[5].Name:= 'MOD';
        Op[0].Priority:= 1; Op[1].Priority:= 1; Op[2].Priority:= 1;
        Op[3].Priority:= 4; Op[4].Priority:= 4; Op[5].Priority:= 4;
        Op[0].Code:= 0; Op[1].Code:= 1; Op[2].Code:= 2;
        Op[3].Code:= 3; Op[4].Code:= 4; Op[5].Code:= 5;
        OpCount:= 20;
    End;
End;

Procedure Solve(Expression: String; UsingDegrees_: Boolean);
Begin
    With App Do Begin
        UsingDegrees:= UsingDegrees_;
        Expr:= AnsiUpperCase(Expression + Chr(13));
        If UsingRPol Then
            GetRPolStr
        Else
            ConvertToRPol;
    End;
End;

Initialization
Begin
    DecimalSeparator:= '.';
    With App Do Begin
        SymbOp[0].Init;
        Op[0].Init;
        UnOp[0].Init;
        Instr[0].Init;
        Func[0].Init;
        VarCount:= 0;
        StrCount:= 0;
        UserCC:= 0;
        UsingDegrees:= True;
        UsingRPol:= False;
    End;
End;
End.
